import { ApolloClient, ApolloLink, HttpLink, InMemoryCache } from '@apollo/client'
import cacheOptions from '../graphql/fragmentTypes.json'

function omitTypename(key: string, value: string) {
  return key === '__typename' ? undefined : value
}

function createOmitTypenameLink() {
  return new ApolloLink((operation, forward) => {
    if (operation.variables) {
      operation.variables = JSON.parse(JSON.stringify(operation.variables), omitTypename)
    }

    return forward(operation)
  })
}

export function createApolloClient(getToken: () => string | Promise<string> | undefined) {
  const cache = new InMemoryCache(cacheOptions)
  const httpLink = new HttpLink({
    uri: `/api`,
    fetch: async (uri, options: { headers: { Authorization: string } }) => {
      const accessToken = await getToken()
      options.headers.Authorization = `Bearer ${accessToken}`
      return fetch(uri, options)
    },
  })
  return new ApolloClient({
    link: ApolloLink.from([createOmitTypenameLink(), httpLink]),
    cache,
  })
}
